/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus MathML Template Set Getter.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const abacusTemplateSetGetter = {
  /* Checking functions
     These functions will recursively call each other as necessary to establish a template pattern.
     They will pass around a templateMatches array to help each other out.
  */

  start: function(aRootIndexNode) {
    var errMsg;
    var walker = aRootIndexNode.ownerDocument.createTreeWalker(aRootIndexNode, NodeFilter.SHOW_ELEMENT,
                                                               editor.hasMEditIndexButIsNotMEditElementFilter, true);
    errMsg = "The root index node passed to this function must have a mEdit:index attribute, but not itself be an ";
    errMsg+= "element in the mEdit namespace.";
    assert(walker.filter.acceptNode(aRootIndexNode) == NodeFilter.FILTER_ACCEPT, errMsg, true);
    var elementsToCheck = [];
    do {
      elementsToCheck.push(walker.currentNode);
    } while (walker.nextNode());
    var templatesList = [];
    var rv = false;
    errMsg = "Failed to retrieve a template for an element! ";
    for (var i = 0; i < elementsToCheck.length; i++) {
      var aNode = elementsToCheck[i];
      var aNodeChildren = editor.getChildElements(aNode);
      if (aNodeChildren.length == 0) {
        continue;
      }
      rv = this.getTemplatePattern(elementsToCheck[i], templatesList);
      assert(rv, errMsg + elementsToCheck[i].getAttributeNS(MEDIT_NS, "index"), true);
    }
    return templatesList;
  },

  getTemplatePattern: function(aIndexNode, templatesList) {
    var rv = false;
    var i;
    var aTemplate;
    var index;
    var rvObject;
    var errMsg = "Matched a template to an index in templatesList that has already been filled!";
    for (i = 0; i < editor.templatesTable.rows.length; i++) {
      aTemplate = editor.templatesTable.rows[i];
      if (!aTemplate.isContentTemplate) {
        continue;
      }
      if (aTemplate.isBase) {
        continue;
      }
      // we want only content non-base templates here
      rvObject = this.indexNodeMatchesTemplate(aIndexNode, aTemplate);
      if (rvObject.condition) {
        index = aIndexNode.getAttributeNS(MEDIT_NS, "index") * 1;
        assert(typeof templatesList[index] == "undefined", errMsg, true);
        templatesList[index] = {
          template: aTemplate,
          count: rvObject.count,
          toString: function() {
            return "[object aTemplatePattern (" + this.template.dialogTitle + ", " + this.count + ")]";
          }
        }
        rv = true;
        break;
      }
    }
    return rv;
  },

  indexNodeMatchesTemplate: function(aIndexNode, aTemplate) {
    var rv = this.checkNode(aIndexNode, aTemplate.contentNode, false, {count: 0});
    return rv;
  },

  checkNode: function(aIndexNode, aRefNode, canSubstitute, rv) {
    var aIndexChildren;
    var aRefChildren;
    var aIndexPointer;

    if (canSubstitute) {
      if ((aRefNode.namespaceURI == MEDIT_NS)&&(aRefNode.localName == "fitb")) {
        if (editor.hasMEditIndexFilter.acceptNode(aIndexNode) == NodeFilter.FILTER_ACCEPT) {
          // It would be nice to make this all into one line, but that's pushing it
          rv.condition = true;
          rv.count++;
          return rv;
        }
      }
    }

    rv.condition = (aRefNode.namespaceURI == aIndexNode.namespaceURI);
    rv.condition = ((rv.condition)&&(aRefNode.localName == aIndexNode.localName));
    if (rv.condition) {
      aIndexChildren = editor.getChildElements(aIndexNode);
      aIndexPointer = 0;
      aRefChildren = editor.getChildElements(aRefNode);
      rv = this.checkChildElements(aIndexChildren, aIndexPointer, aRefChildren, rv);
      if ((rv.condition)&&(editor.hasMEditIndexFilter.acceptNode(aIndexNode) == NodeFilter.FILTER_ACCEPT)) {
        rv.count++;
      }
    }
    return rv;
  },

  checkChildElements: function(aIndexArray, aIndexPointer, aRefArray, rv) {
    var aRefPointer;
    var aRefNode;
    var aIndexNode;
    var aSetChildren;
    var aSetCount;
    var i;

    if (typeof aIndexArray[aIndexPointer] == "undefined") {
      rv.condition = true;
      return rv;
    }

    for (aRefPointer = 0; aRefPointer < aRefArray.length; aRefPointer++) {
      aRefNode = aRefArray[aRefPointer];
      aIndexNode = aIndexArray[aIndexPointer];
      if (aRefNode.namespaceURI == MEDIT_NS) {
        if (aRefNode.localName == "fitb") {
          rv = this.checkNode(aIndexNode, aRefNode, true, rv);
          if (!rv.condition) {
            break;
          }
        }

        if (aRefNode.localName == "fitb-set") {
          aSetChildren = editor.getChildElements(aRefNode);
          aSetCount = 0;
          var foo = {count: 0};
          foo = this.checkChildElements(aIndexArray, aIndexPointer, aSetChildren, foo);
          while (foo.condition) {
            aSetCount++;
            if (typeof aIndexArray[aIndexPointer + aSetCount * aSetChildren.length] == "undefined") {
              break;
            }
            foo = {count: 0};
            foo = this.checkChildElements(aIndexArray, aIndexPointer + aSetCount * aSetChildren.length, aSetChildren, foo);
          }

/*
          if ((aSetCount < aRefNode.min)||(aSetCount > aRefNode.max)) {
            rv.condition = false;
            break;
          }
*/

          for (i = 0; i < aSetCount * aSetChildren.length; i++) {
            if (editor.hasMEditIndexFilter.acceptNode(aIndexArray[aIndexPointer + i]) == NodeFilter.FILTER_ACCEPT) {
              rv.count++;
            }
          }
          aIndexPointer += (aSetCount * aSetChildren.length);
          continue;
        }
      // end aRefNode.namespaceURI == MEDIT_NS;
      } else {
        rv = this.checkNode(aIndexNode, aRefNode, false, rv);
        if (!rv.condition) {
          break;
        }
      }
      aIndexPointer++;
    } // end for (aRefPointer = 0...);

    return rv;
  },

  toString: function() {
    return "[object abacusTemplateSetGetter]";
  }
};
